package com.nisum.poc.api.employee.service.impl;

import com.nisum.poc.api.employee.dto.EmployeeDto;
import com.nisum.poc.api.employee.entity.Employee;
import com.nisum.poc.api.employee.repository.EmployeeProfileRepository;
import com.nisum.poc.api.employee.response.EmployeeProfileResponse;
import com.nisum.poc.api.employee.service.IEmployeeProfileService;
import org.modelmapper.ModelMapper;
import org.modelmapper.convention.MatchingStrategies;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@Service
public class EmployeeProfileService implements IEmployeeProfileService {


    private EmployeeProfileRepository employeeProfileRepository;

    @Autowired
    public EmployeeProfileService(EmployeeProfileRepository employeeProfileRepository){
        this.employeeProfileRepository = employeeProfileRepository;
    }


    /**
     *
     * @param employeeDto
     * @return
     */
    @Override
    public Mono<Employee> createEmployeeProfile(EmployeeDto employeeDto) {


      return employeeProfileRepository.insert(serviceDtoConvertor(employeeDto));
    }

    @Override
    public Flux<Employee> findAllEmployee() {
        return employeeProfileRepository.findAll();
    }

    /**
     *
     * @param empId
     * @return
     */
    @Override
    public Mono<Employee> fetchEmployeeDetailsById(Long empId) {
        return employeeProfileRepository.findById(empId);
    }

    /**
     *
     * @param employeeDto
     */
    private Employee serviceDtoConvertor(EmployeeDto employeeDto){

        ModelMapper modelMapper = new ModelMapper();
        modelMapper.getConfiguration().setMatchingStrategy(MatchingStrategies.STRICT);
        return  modelMapper.map(employeeDto, Employee.class);
    }


    /**
     *
     */
    private EmployeeProfileResponse convertToResponse(Employee employee){

        ModelMapper modelMapper = new ModelMapper();
        modelMapper.getConfiguration().setMatchingStrategy(MatchingStrategies.STRICT);
        return  modelMapper.map(employee, EmployeeProfileResponse.class);
    }
}
